import os
import copy

import data
import colors
from actor import Actor
from water import WaterActor
from infoactor import InfoActor
from player import Player
from mathactor import MathActor
from guielement import GuiElement
from guielement import TextElement
from collisionsystem import Point
from sprite import SpriteManager
from inputgame import TextReader
from mathcomponent import MathComponent
from editcomponent import EditComponent
from menucomponent import MenuComponent

class DataManager:
	def __init__(self, w, h):
		self.w = w
		self.h = h
		self.keepRunning = True
		self.frameCount = 0
		self.targetGameSpeed = 1.0
		self.gameSpeed = 1.0
		
		self.spriteManager = SpriteManager()

		self.fonts = {"menuFont":("lousy.ttf", 60),
					  "mainFont":("lousy.ttf", 30),
					  "mathFont":("lousy.ttf", 55)}
		
		self.aliveActors = []
		self.waterActors = []
		self.actorID = 0
		self.playerActor = None
		
		self.guiElements = []
		self.textElements = []
		self.textReader = TextReader()
		
		self.mathComponent  = MathComponent(self)

		self.inMainScreen = True
		self.inLevelEdit = False
		self.editComponent = EditComponent(self)
		self.camera = Point(0,0)
		
		self.currentLevel = None
		self.levels = ["mainscreen"]
		f = open(data.filepath("game.txt"))
		for line in f.readlines():
			if line[0] != "#":
				self.levels.append(line.strip())
		
		self.sound = None
		
		self.menuComponent = MenuComponent(self)
		self.menuComponent.enterMenu()
		
	def setInMainScreen(self, b):
		self.inMainScreen = b
		if b and self.menuComponent is not None: 
			self.menuComponent.enterMenu()
		elif not b and self.menuComponent is not None:
			self.menuComponent.leaveMenu()
		
	def clear(self):	
		self.frameCount = 0
		self.targetGameSpeed = 1.0
		self.gameSpeed = 1.0
		self.camera = Point(0,0)
		
		self.aliveActors = []
		self.waterActors = []
		self.actorID = 0
		self.playerActor = None
		
		self.guiElements = []
		self.textElements = []
		self.inLevelEdit = False
		
		self.gameOverText  = self.createText("1", "mainFont", colors.black, Point(-100,0), " Game Over! ")
		self.gameOverText2 = self.createText("1", "mainFont", colors.black, Point(-120,-50), " Try Again? (Y/N) ")
		self.levelCompletedText = self.createText("1", "mainFont", colors.black, Point(-100,0), " Level Complete! ")
		self.levelCompletedText2 = self.createText("1", "mainFont", colors.black, Point(-100,-50), " (Press Enter..) ")
		
		self.sound.clear()
		
		self.menuComponent.clear()

	def getNextLevel(self):
		i = self.levels.index(self.currentLevel.name)
		if i == len(self.levels) - 1:
			return "mainscreen"
		else:
			return self.levels[i+1]

	
	def toggleEdit(self):
		self.inLevelEdit = not self.inLevelEdit
		if self.inLevelEdit:
			self.editGui = self.createGui("EditActive", Point(0,250), 0.0, "cloud1.png")
		else:
			self.removeGui(self.editGui)
		
	def playerCollided(self):
		self.mathComponent.deActivate()

	def removeText(self, e):
		try:
			self.textElements.remove(e)
		except:
			pass

	def createText(self, name, font, color, pos, text):
		e = TextElement( name, font, color, pos, text )
		self.textElements.append(e)
		return e

	def removeGui(self, g):
		for child_g in g.elements: 
			self.removeGui(child_g)
		for child_t in g.texts: 
			self.removeText(child_t)
		try:
			self.guiElements.remove(g)
		except:
			pass

	def createGui(self, name, pos, z, image):
		if image is not None:
			sprite = self.spriteManager.create(image)
		else:
			sprite = None
		e = GuiElement( name, pos, z, sprite )
		self.guiElements.append(e)
		return e

	def removeActor(self, actor):
		try:
			self.aliveActors.remove(actor)
		except:
			pass
		try:
			self.waterActors.remove(actor)
		except:
			pass

	def createActor(self, name, pos, image, info = None):
		sprite = self.spriteManager.create(image)
		if name == "player":
			a = Player( self.actorID, Point(0,0), sprite, self)
			self.playerActor = a
			self.playerActor.speed = Point(50,20)
		elif name == "powerup":
			a = MathActor(self.actorID, Point(0,0), sprite, self )
			a.setMathSymbol(info)
		elif name == "info":
			a = InfoActor(self.actorID, Point(0,0), sprite, self )
			a.infotext = info.replace("_", " ")
		elif name == "fluff":
			a = Actor( self.actorID, Point(0,0), sprite, self )
			
			if info is not None:
				#print name, pos, image, info
				#print a.sprite.anims
				a.sprite.setAnimation(info)
			a.stuck = True
			a.doCollisionTest = False			
		elif name == "water":
			a = WaterActor( self.actorID, Point(0,0), sprite, self)
			self.waterActors.append(a)
		elif name == "bg":
			a = Actor( self.actorID, Point(0,0), sprite, self )
			a.moveWithCamera = False
			a.aabb = None
			a.stuck = True
		else:
			a = Actor( self.actorID, Point(0,0), sprite, self )
			a.stuck = True
		
		if name == "integral":
			a.type = "Integral"
			
		try:
			a.aabb = copy.copy(self.spriteManager.spriteAABBs[image])
			a.aabb.owner = a
		except:
			# no bb, which is probably fine
			#print "No aabb for image", image
			pass
		
		a.move(pos)
		self.actorID += 1
		self.aliveActors.append(a)

				
		return a
		
	def getDistToClosestActor(self, point):
		nearest = None
		nearestdist = 1000000
		for actor in self.aliveActors:
			vec = Point(point.x - actor.pos.x, point.y - actor.pos.y)
			dist = vec.length()
			if dist < nearestdist:
				nearestdist = dist
				nearest = actor
				
		return nearestdist

    
    
    