from collisionsystem import Point, AABB

FRAMES = 0
LOOPING = 1

class Sprite:
	def __init__(self, image, size, framesize, animtime = 10000.0):
		self.image = image
		self.size = size
		self.frameSize = framesize
		self.animTime = animtime
		
		self.index = Point(0, 0)
		self.currentTime = 0.0
		
		self.anims = {None: ([0], False)} # ([1,2,3], True) -- (frames, isLooping)
		self.currentAnim = None
		self.currentFrame = 0
		
		self.quad = None;
		self.calcQuad()
	
		self.color = (1.0, 1.0, 1.0, 1.0)
		
	def setAnimation(self, name):	
		if self.currentAnim == name and self.anims[name][LOOPING]:
			### We dont reset animations that loops..
			return
		
		self.currentAnim = name
		self.currentFrame = 0
		self.setIndex(self.anims[self.currentAnim][FRAMES][self.currentFrame])
		self.calcQuad()
		
	def setIndex(self, frame):
		self.index.x = frame % (self.size.x / self.frameSize.x)
		self.index.y = frame / (self.size.x / self.frameSize.x)
		
	def update(self, frametime):
		if self.animTime == 0.0:
			return
		
		self.currentTime += frametime
		if self.currentTime > self.animTime:
			self.currentTime = 0.0
			self.nextFrame()
			self.calcQuad()
			
	def nextFrame(self):
		#print self.currentFrame
		frames = self.anims[self.currentAnim][FRAMES]
		looping = self.anims[self.currentAnim][LOOPING]
		
		if self.currentFrame >= len(frames)-1:
			if looping: ### Anim is done, lets loop
				self.currentFrame = 0
				self.setIndex(frames[self.currentFrame])
		else:
			self.currentFrame += 1
			self.setIndex(frames[self.currentFrame])
			
	def calcQuad(self):
		### unnecessary to do this each time...
		top    = 1-self.frameSize.y * (self.index.y + 1) / float(self.size.y)
		bottom = 1-self.frameSize.y * (self.index.y + 0) / float(self.size.y)
		left   = self.frameSize.x * (self.index.x + 0) / float(self.size.x)
		right  = self.frameSize.x * (self.index.x + 1) / float(self.size.x)

		self.quad = []
		self.quad.append( Point( left, bottom ) )
		self.quad.append( Point( right, bottom ) )
		self.quad.append( Point( right, top ) )
		self.quad.append( Point( left, top ) )
		#return quad
	
	def getQuad(self):
		return self.quad
		
		
class SpriteManager:
	def __init__(self):
		self.spriteCreators = {}
		self.spriteCreators["theta.png"] = self.createTheta
		self.spriteCreators["cloud1.png"] = self.createMessageCloud
		self.spriteCreators["mathbg.png"] = self.createMathBG
		self.spriteCreators["star.png"] = self.createStar
		self.spriteCreators["water.png"] = self.createWater
		self.spriteCreators["integral.png"] = self.createIntegral
		self.spriteCreators["info.png"] = self.createInfo
		
		### Ground
		self.spriteCreators["ground_256.png"] = self.createGround256
		self.spriteCreators["ground_64.png"] = self.createGround64
		self.spriteCreators["stone_128.png"] = self.createStone128
		self.spriteCreators["stone_64.png"] = self.createStone64
		
		### Fluff
		self.spriteCreators["bush_64.png"] = self.createBush64
		self.spriteCreators["tree_512.png"] = self.createTree512
		self.spriteCreators["flower_256.png"] = self.createFlower256
		self.spriteCreators["mush_256.png"] = self.createMush256
		
		self.spriteIDs = {}
		
		self.spriteAABBs = {}
		self.spriteAABBs["theta.png"]      	= AABB( Point(-32+15, 32-13), Point(32-15, -32))
		self.spriteAABBs["star.png"] 		= AABB( Point(-32, 32), Point(32,-32))
		self.spriteAABBs["tree512.png"] 		= AABB( Point(10000, 10000), Point(10000,10000))

		self.spriteAABBs["ground_256.png"] 	= AABB( Point(-128+10, 15), Point(128-10,-15))
		self.spriteAABBs["ground_64.png"] 		= AABB( Point(-32+5, 15), Point(32-5,-15))
		self.spriteAABBs["stone_128.png"] 		= AABB( Point(-32+2, 64-2), Point(32+2,-64-2))
		self.spriteAABBs["stone_64.png"] 		= AABB( Point(-32+2, 32-2), Point(32+2,-32-2))
		self.spriteAABBs["water.png"] 		    = AABB( Point(-1024, 128), Point(1024, -128))
		self.spriteAABBs["integral.png"] 		= AABB( Point(-64, 128), Point(0, -128))
	
	def create(self, name):
		if name.find("levelbg") != -1:
			return self.createLevelBG( name )
		else:
			return self.spriteCreators[name]()
	
	def createLevelBG(self, name):
		#return Sprite(name, Point(800, 600), Point(800, 600) )
		return Sprite(name, Point(800, 800), Point(800, 800) )
			
	def createMathBG(self):
		return Sprite("mathbg.png", Point(400, 300), Point(400, 300) )
	
	def createTheta(self):
		s = Sprite("theta.png", Point(512, 64), Point(64, 64), 0.1 )
		s.anims["stand"] = ([0,0,4,4], True)
		s.anims["run"] = ([0,1,2,1,0,3], True)
		s.anims["jump"] = ([5], False)
		s.setAnimation("stand")
		return s
	
	def createStar(self):
		s = Sprite("star.png", Point(1024, 64), Point(64,64))
		s.anims["1"] = ([0], False)
		s.anims["2"] = ([1], False)
		s.anims["3"] = ([2], False)
		s.anims["4"] = ([3], False)
		s.anims["5"] = ([4], False)
		s.anims["6"] = ([5], False)
		s.anims["7"] = ([6], False)
		s.anims["8"] = ([7], False)
		s.anims["9"] = ([8], False)	
		return s
	
	def createMessageCloud(self):
		return Sprite("cloud1.png", Point(256, 128), Point(256, 128))
	
	def createIntegral(self):
		return Sprite("integral.png", Point(128, 256), Point(128, 256) )
		#s = Sprite("integral.png", Point(256, 256), Point(64, 256), 0.1)
		#s.anims["integrate"] = ([0,1,2,3,3,2,1,0], True)
		#s.setAnimation("integrate")
		#return s
	
	def createInfo(self):
		return Sprite("info.png", Point(64, 64), Point(64, 64) )
	
	### Ground
	def createGround256(self):
		return Sprite("ground_256.png", Point(256, 32), Point(256,32))

	def createGround64(self):
		return Sprite("ground_64.png", Point(64, 32), Point(64,32))
	
	def createStone128(self):
		return Sprite("stone_128.png", Point(64, 128), Point(64,128))
	def createStone64(self):
		return Sprite("stone_64.png", Point(64, 64), Point(64,64))
	
	### Fluff
	def createBush64(self):
		s = Sprite("bush_64.png", Point(64, 32), Point(64,32))
		s.anims["1"] = ([0], False)
		return s
	
	def createFlower256(self):
		s =  Sprite("flower_256.png", Point(256, 64), Point(64, 64))
		s.anims["1"] = ([0], False)
		s.anims["2"] = ([1], False)
		s.anims["3"] = ([2], False)
		s.anims["4"] = ([3], False)
		return s
	def createMush256(self):
		s =  Sprite("mush_256.png", Point(512, 128), Point(128, 128))
		s.anims["1"] = ([0], False)
		s.anims["2"] = ([1], False)
		s.anims["3"] = ([2], False)
		s.anims["4"] = ([3], False)
		return s
	
	def createTree512(self):
		s =  Sprite("tree_512.png", Point(512, 128), Point(128, 128))
		s.anims["1"] = ([0], False)
		s.anims["2"] = ([1], False)
		s.anims["3"] = ([2], False)
		s.anims["4"] = ([3], False)
		return s
	
	def createWater(self):
		return Sprite("water.png", Point(2048,512), Point(2048,512))
	